@@ -34,6 +34,7 @@
 
 #ifdef HAVE_LIBPURPLE
 #include "fix_purple.h"
+#include "utils/msgpluslive.h"
 
 /* libpurple stuff. */
 #include "fix_purple_win32.h"
@@ -89,7 +90,7 @@ pecan_contact_update (PecanContact *contact)
 
     g_return_if_fail (contact->contactlist);
 
-    account = msn_session_get_account (contact->contactlist->session);
+    account = msn_session_get_user_data (contact->contactlist->session);
 
     if (contact->status)
         purple_prpl_got_user_status (account, contact->passport, contact->status, NULL);
@@ -193,23 +194,35 @@ pecan_contact_set_friendly_name (PecanContact *contact,
         return;
     }
 
-    g_free (contact->friendly_name);
-    contact->friendly_name = g_strdup (name);
-
 #ifdef HAVE_LIBPURPLE
+    PurpleAccount *account;
+    account = msn_session_get_user_data (contact->contactlist->session);
+
     g_return_if_fail (contact->contactlist);
 
+    if (purple_account_get_bool (account, "hide_msgplus_tags", TRUE))
     {
-        PurpleAccount *account;
-        PurpleConnection *gc;
-        MsnSession *session;
+        char* parsed_name;
 
-        session = contact->contactlist->session;
-        account = msn_session_get_account (session);
-        gc = purple_account_get_connection (account);
+        parsed_name = remove_plus_tags_from_str (name);
 
-        purple_buddy_set_nickname (gc, contact->passport, contact->friendly_name);
+        if (!parsed_name)
+            parsed_name = g_strdup (name);
+        if (contact->friendly_name && parsed_name &&
+            strcmp (contact->friendly_name, parsed_name) == 0)
+            return;
+
+        g_free (contact->friendly_name);
+        contact->friendly_name = parsed_name;
     }
+    else
+    {
+        g_free (contact->friendly_name);
+        contact->friendly_name = g_strdup (name);
+    }
+
+    purple_buddy_set_nickname (purple_account_get_connection (account),
+                               contact->passport, contact->friendly_name);
 
     /** @todo temporarily disable this until we have proper server-side aliases
      * support. */
@@ -222,6 +235,9 @@ pecan_contact_set_friendly_name (PecanContact *contact,
         pecan_contact_set_store_name (contact, name);
     }
 #endif
+#else
+    g_free (contact->friendly_name);
+    contact->friendly_name = g_strdup (name);
 #endif /* HAVE_LIBPURPLE */
 }
 
@@ -239,8 +255,36 @@ pecan_contact_set_personal_message (PecanContact *contact,
         return;
     }
 
+#ifdef HAVE_LIBPURPLE
+    PurpleAccount *account;
+    account = msn_session_get_user_data (contact->contactlist->session);
+
+    g_return_if_fail (contact->contactlist);
+
+    if (value && purple_account_get_bool (account, "hide_msgplus_tags", TRUE))
+    {
+        char* parsed_value;
+
+        parsed_value = remove_plus_tags_from_str (value);
+
+        if (!parsed_value)
+            parsed_value = g_strdup (value);
+        if (contact->personal_message && parsed_value &&
+            strcmp (contact->personal_message, parsed_value) == 0)
+            return;
+
+        g_free (contact->personal_message);
+        contact->personal_message = parsed_value;
+    }
+    else
+    {
+        g_free (contact->personal_message);
+        contact->personal_message = g_strdup (value);
+    }
+#else
     g_free (contact->personal_message);
     contact->personal_message = g_strdup (value);
+#endif /* HAVE_LIBPURPLE */
 }
 
 void
@@ -285,14 +329,14 @@ pecan_contact_set_store_name (PecanContact *contact,
 
     {
         PurpleAccount *account;
-        PurpleConnection *gc;
+        PurpleConnection *connection;
         MsnSession *session;
 
         session = contact->contactlist->session;
-        account = msn_session_get_account (session);
-        gc = purple_account_get_connection (account);
+        account = msn_session_get_user_data (session);
+        connection = purple_account_get_connection (account);
 
-        purple_buddy_set_displayname (gc, contact->passport, contact->store_name);
+        purple_buddy_set_displayname (connection, contact->passport, contact->store_name);
     }
 
     /** @todo temporarily disable this until we have proper server-side aliases
@@ -431,7 +475,7 @@ pecan_contact_add_group_id (PecanContact *contact,
 
         contactlist = contact->contactlist;
         group_name = pecan_contactlist_find_group_name (contactlist, group_guid);
-        account = msn_session_get_account (contactlist->session);
+        account = msn_session_get_user_data (contactlist->session);
 
         /* If this contact is in the no-group, remove him, since now he is in a
          * group. */
