@@ -29,6 +29,7 @@
 #include "session.h"
 #include "pecan_util.h"
 #include "pecan_printf.h"
+#include "pecan_log.h"
 
 #include <string.h>
 
@@ -57,7 +58,7 @@ msn_slp_call_new(MsnSlpLink *slplink)
 	slpcall = g_new0(MsnSlpCall, 1);
 
 #ifdef PECAN_DEBUG_SLPCALL
-	purple_debug_info("msn", "slpcall_new: slpcall(%p)\n", slpcall);
+	pecan_info ("slpcall_new: slpcall(%p)\n", slpcall);
 #endif
 
 	slpcall->slplink = slplink;
@@ -77,7 +78,7 @@ msn_slp_call_destroy(MsnSlpCall *slpcall)
 	MsnSession *session;
 
 #ifdef PECAN_DEBUG_SLPCALL
-	purple_debug_info("msn", "slpcall_destroy: slpcall(%p)\n", slpcall);
+	pecan_info ("slpcall_destroy: slpcall(%p)\n", slpcall);
 #endif
 
 	g_return_if_fail(slpcall != NULL);
@@ -100,7 +101,7 @@ msn_slp_call_destroy(MsnSlpCall *slpcall)
 		e = e->next;
 
 #ifdef PECAN_DEBUG_SLPCALL_VERBOSE
-		purple_debug_info("msn", "slpcall_destroy: trying slpmsg(%p)\n",
+		pecan_info ("slpcall_destroy: trying slpmsg(%p)\n",
 						slpmsg);
 #endif
 
@@ -205,7 +206,7 @@ msn_slp_call_timeout(gpointer data)
 	slpcall = data;
 
 #ifdef PECAN_DEBUG_SLPCALL
-	purple_debug_info("msn", "slpcall_timeout: slpcall(%p)\n", slpcall);
+	pecan_info ("slpcall_timeout: slpcall(%p)\n", slpcall);
 #endif
 
 	if (!slpcall->pending && !slpcall->progress)
@@ -230,7 +231,7 @@ msn_slp_process_msg(MsnSlpLink *slplink, MsnSlpMessage *slpmsg)
 	body = slpmsg->buffer;
 	body_len = slpmsg->size;
 
-	if (slpmsg->flags == 0x0)
+	if (slpmsg->flags == 0x0 || slpmsg->flags == 0x1000000)
 	{
 		char *body_str;
 
@@ -257,7 +258,7 @@ msn_slp_process_msg(MsnSlpLink *slplink, MsnSlpMessage *slpmsg)
 		}
 		g_free(body_str);
 	}
-	else if (slpmsg->flags == 0x20 || slpmsg->flags == 0x1000030)
+	else if (slpmsg->flags == 0x20 || slpmsg->flags == 0x1000020 || slpmsg->flags == 0x1000030)
 	{
 		slpcall = msn_slplink_find_slp_call_with_session_id(slplink, slpmsg->session_id);
 
@@ -280,6 +281,8 @@ msn_slp_process_msg(MsnSlpLink *slplink, MsnSlpMessage *slpmsg)
 			msn_slp_call_session_init(slpcall);
 	}
 #endif /* MSN_DIRECTCONN */
+	else
+		pecan_warning ("slp_process_msg: unprocessed SLP message with flags 0x%08lx", slpmsg->flags);
 
 	return slpcall;
 }
