@@ -60,6 +60,12 @@
 #include <util.h>
 #include <prefs.h>
 
+#if defined(PECAN_CVR)
+#if PURPLE_VERSION_CHECK(2,5,0)
+#include <smiley.h>
+#endif /* PURPLE_VERSION_CHECK(2,5,0) */
+#endif /* defined(PECAN_CVR) */
+
 #define _Q(x) #x
 #define Q(x) _Q(x)
 #define PLUGIN_ID "prpl-" Q(PLUGIN_NAME)
@@ -71,6 +77,14 @@ typedef struct
 
 } MsnMobileData;
 
+#if PURPLE_VERSION_CHECK(2,5,0)
+typedef struct
+{
+	char *smile;
+	MsnObject *obj;
+} MsnEmoticon;
+#endif /* PURPLE_VERSION_CHECK(2,5,0) */
+
 /* exports */
 void msn_set_friendly_name (PurpleConnection *gc, const gchar *entry);
 
@@ -122,6 +136,7 @@ msn_send_attention(PurpleConnection *gc, const char *username, guint type)
 		return FALSE;
 
 	msn_switchboard_send_msg(swboard, msg, TRUE);
+	msn_message_destroy(msg);
 
 	return TRUE;
 }
@@ -194,6 +209,19 @@ msn_set_prp(PurpleConnection *gc, const char *type, const char *entry)
 	}
 }
 
+#ifndef PECAN_USE_PSM
+static void
+msn_set_personal_message_cb (PurpleConnection *gc, const gchar *entry)
+{
+	MsnSession *session;
+
+	session = gc->proto_data;
+	purple_account_set_string(session->user_data, "personal_message", entry);
+
+	pecan_update_personal_message (session);
+}
+#endif /* PECAN_USE_PSM */
+
 static void
 msn_set_home_phone_cb(PurpleConnection *gc, const char *entry)
 {
@@ -283,6 +311,28 @@ msn_show_set_friendly_name(PurplePluginAction *action)
 					   gc);
 }
 
+#ifndef PECAN_USE_PSM
+static void
+msn_show_set_personal_message (PurplePluginAction *action)
+{
+	PurpleConnection *gc;
+	MsnSession *session;
+
+	gc = (PurpleConnection *) action->context;
+	session = gc->proto_data;
+
+	purple_request_input(gc, NULL, _("Set your personal message."),
+					   _("This is the message that other MSN buddies will "
+						 "see under your name."),
+					   purple_account_get_string (session->user_data, "personal_message", ""),
+					   FALSE, FALSE, NULL,
+					   _("OK"), G_CALLBACK(msn_set_personal_message_cb),
+					   _("Cancel"), NULL,
+					   purple_connection_get_account(gc), NULL, NULL,
+					   gc);
+}
+#endif /* PECAN_USE_PSM */
+
 static void
 msn_show_set_home_phone(PurplePluginAction *action)
 {
@@ -605,10 +655,14 @@ util_gen_state (PurpleStatusPrimitive primitive,
                 const gchar *id,
                 const gchar *name)
 {
+#ifdef PECAN_USE_PSM
     return purple_status_type_new_with_attrs (primitive,
                                               id, name, TRUE, TRUE, FALSE,
                                               "message", _("Message"), purple_value_new (PURPLE_TYPE_STRING),
                                               NULL);
+#else
+    return purple_status_type_new_full (primitive, id, name, TRUE, TRUE, FALSE);
+#endif /* PECAN_USE_PSM */
 }
 
 static GList *
@@ -653,10 +707,19 @@ msn_actions(PurplePlugin *plugin, gpointer context)
 	GList *m = NULL;
 	PurplePluginAction *act;
 
+        session = gc->proto_data;
+
 	act = purple_plugin_action_new(_("Set Friendly Name..."),
 								 msn_show_set_friendly_name);
 	m = g_list_append(m, act);
-	m = g_list_append(m, NULL);
+
+#ifndef PECAN_USE_PSM
+	act = purple_plugin_action_new(_("Set Personal Message..."),
+					msn_show_set_personal_message);
+	m = g_list_append(m, act);
+#endif /* PECAN_USE_PSM */
+
+        m = g_list_append(m, NULL);
 
 	act = purple_plugin_action_new(_("Set Home Phone Number..."),
 								 msn_show_set_home_phone);
@@ -681,7 +744,6 @@ msn_actions(PurplePlugin *plugin, gpointer context)
 			msn_show_set_mobile_pages);
 	m = g_list_append(m, act);
 
-        session = gc->proto_data;
 	user = msn_session_get_username(session);
 
 	if ((strstr(user, "@hotmail.") != NULL) ||
@@ -760,7 +822,9 @@ login (PurpleAccount *account)
     host = purple_account_get_string (account, "server", MSN_SERVER);
     port = purple_account_get_int (account, "port", MSN_PORT);
 
-    session = msn_session_new (account);
+    session = msn_session_new (purple_account_get_username (account),
+                               purple_account_get_password (account),
+                               purple_account_get_bool (account, "http_method", FALSE));
 
     gc->proto_data = session;
     gc->flags |= PURPLE_CONNECTION_HTML | \
@@ -769,10 +833,14 @@ login (PurpleAccount *account)
                  PURPLE_CONNECTION_NO_FONTSIZE | \
                  PURPLE_CONNECTION_NO_URLDESC;
 
-    msn_session_set_login_step (session, PECAN_LOGIN_STEP_START);
+#if PURPLE_VERSION_CHECK(2,5,0)
+    gc->flags |= PURPLE_CONNECTION_ALLOW_CUSTOM_SMILEY;
+#endif /* PURPLE_VERSION_CHECK(2,5,0) */
 
-    msn_session_set_username (session, purple_account_get_username (account));
-    msn_session_set_password (session, purple_account_get_password (account));
+    session->user_data = account;
+    session->server_alias = purple_account_get_bool (account, "server_alias", FALSE);
+    session->use_directconn = purple_account_get_bool (account, "use_directconn", FALSE);
+    session->use_userdisplay = purple_account_get_bool (account, "use_userdisplay", FALSE);
 
     if (!msn_session_connect (session, host, port))
         purple_connection_error (gc, _("Failed to connect to server."));
@@ -792,6 +860,108 @@ logout (PurpleConnection *gc)
     gc->proto_data = NULL;
 }
 
+#if defined(PECAN_CVR)
+#if PURPLE_VERSION_CHECK(2,5,0)
+static GString*
+msn_msg_emoticon_add(GString *current, MsnEmoticon *emoticon)
+{
+	MsnObject *obj;
+	char *strobj;
+
+	if (emoticon == NULL)
+            return current;
+
+	obj = emoticon->obj;
+
+	if (!obj)
+            return current;
+
+	strobj = msn_object_to_string(obj);
+
+	if (current)
+	{
+		g_string_append_printf(current, "\t%s\t%s", emoticon->smile, strobj);
+	}
+	else
+	{
+		current = g_string_new("");
+		g_string_printf(current,"%s\t%s", emoticon->smile, strobj);
+	}
+
+	g_free(strobj);
+
+	return current;
+}
+
+static void
+msn_send_emoticons(MsnSwitchBoard *swboard, GString *body)
+{
+	MsnMessage *msg;
+
+	g_return_if_fail(body != NULL);
+
+	msg = msn_message_new(MSN_MSG_SLP);
+	msn_message_set_content_type(msg, "text/x-mms-emoticon");
+	msn_message_set_flag(msg, 'N');
+	msn_message_set_bin_data(msg, body->str, body->len);
+
+	msn_switchboard_send_msg(swboard, msg, TRUE);
+	msn_message_destroy(msg);
+}
+
+static void
+msn_emoticon_destroy(MsnEmoticon *emoticon)
+{
+	if (emoticon->obj)
+            msn_object_destroy(emoticon->obj);
+	g_free(emoticon->smile);
+	g_free(emoticon);
+}
+
+static GSList *
+msn_msg_grab_emoticons(const char *msg, const char *username)
+{
+	GSList *list;
+	GList *smileys;
+	PurpleSmiley *smiley;
+	PurpleStoredImage *image;
+	char *ptr;
+	MsnEmoticon *emoticon;
+	int length;
+
+	list = NULL;
+	smileys = purple_smileys_get_all();
+	length = strlen(msg);
+
+	for (; smileys; smileys = g_list_delete_link(smileys, smileys))
+	{
+            PecanBuffer *buffer;
+            smiley = smileys->data;
+
+            ptr = g_strstr_len(msg, length, purple_smiley_get_shortcut(smiley));
+
+            if (!ptr)
+                continue;
+
+            image = purple_smiley_get_stored_image(smiley);
+            buffer = pecan_buffer_new_memdup ((const gpointer) purple_imgstore_get_data (image),
+                                              purple_imgstore_get_size (image));
+
+            emoticon = g_new0(MsnEmoticon, 1);
+            emoticon->smile = g_strdup(purple_smiley_get_shortcut(smiley));
+            emoticon->obj = msn_object_new_from_image(buffer,
+                                                      purple_imgstore_get_filename(image),
+                                                      username, MSN_OBJECT_EMOTICON);
+
+            purple_imgstore_unref(image);
+            list = g_slist_prepend(list, emoticon);
+	}
+
+	return list;
+}
+#endif /* PURPLE_VERSION_CHECK(2,5,0) */
+#endif /* defined(PECAN_CVR) */
+
 static gint
 send_im (PurpleConnection *gc,
          const gchar *who,
@@ -846,6 +1016,41 @@ send_im (PurpleConnection *gc,
 
             swboard = msn_session_get_swboard (session, who, MSN_SB_FLAG_IM);
 
+#if defined(PECAN_CVR)
+#if PURPLE_VERSION_CHECK(2,5,0)
+            const char *username;
+
+            username = msn_session_get_username (session);
+
+            if (g_ascii_strcasecmp (who, username) != 0)
+            {
+                MsnEmoticon *smile;
+                GSList *smileys;
+                GString *emoticons = NULL;
+
+                pecan_debug ("send via switchboard");
+                smileys = msn_msg_grab_emoticons (message, username);
+
+                while (smileys)
+                {
+                    smile = (MsnEmoticon*)smileys->data;
+                    emoticons = msn_msg_emoticon_add(emoticons, smile);
+                    msn_emoticon_destroy(smile);
+                    smileys = g_slist_delete_link(smileys, smileys);
+                }
+
+                if (emoticons)
+                {
+                    msn_send_emoticons(swboard, emoticons);
+                    g_string_free(emoticons, TRUE);
+                }
+            }
+#endif /* PURPLE_VERSION_CHECK(2,5,0) */
+#endif /* defined(PECAN_CVR) */
+
+            if (flags & PURPLE_MESSAGE_AUTO_RESP)
+                msn_message_set_flag (msg, 'U');
+
             msn_switchboard_send_msg (swboard, msg, TRUE);
         }
 
@@ -909,7 +1114,9 @@ set_status (PurpleAccount *account,
     {
         session = gc->proto_data;
         pecan_update_status (session);
+#ifdef PECAN_USE_PSM
         pecan_update_personal_message (session);
+#endif /* PECAN_USE_PSM */
     }
 }
 
@@ -1520,9 +1727,9 @@ static PurplePluginProtocolInfo prpl_info =
     NULL, /* unregister_user */
     msn_send_attention, /* send_attention */
     msn_attention_types, /* attention_types */
-#ifdef LIBPURPLE_NEW_API
+#if PURPLE_VERSION_CHECK(2,5,0)
     sizeof (PurplePluginProtocolInfo), /* struct_size */
-#endif
+#endif /* PURPLE_VERSION_CHECK(2,5,0) */
     /* padding */
     NULL
 };
@@ -1588,6 +1795,9 @@ init_plugin (PurplePlugin *plugin)
 
         option = purple_account_option_bool_new (_("Use user displays"), "use_userdisplay", TRUE);
         prpl_info.protocol_options = g_list_append (prpl_info.protocol_options, option);
+
+        option = purple_account_option_bool_new (_("Don't show Messenger Plus! tags"), "hide_msgplus_tags", TRUE);
+        prpl_info.protocol_options = g_list_append (prpl_info.protocol_options, option);
     }
 
     purple_cmd_register ("nudge", "", PURPLE_CMD_P_PRPL,
