@@ -65,6 +65,7 @@ typedef enum
 	MSN_CLIENT_CAP_WEBMSGR    = 0x00100,
 	MSN_CLIENT_CAP_DIRECTIM   = 0x04000,
 	MSN_CLIENT_CAP_WINKS      = 0x08000,
+	MSN_CLIENT_CAP_VOICE_CLIP = 0x40000,
 	MSN_CLIENT_CAP_SEARCH     = 0x10000
 
 } MsnClientCaps;
@@ -84,7 +85,11 @@ typedef enum
 #define MSN_CLIENT_ID_RESERVED_1   0x00
 #define MSN_CLIENT_ID_RESERVED_2   0x00
 #if defined(PECAN_CVR)
+#if defined(PECAN_LIBSIREN)
+#define MSN_CLIENT_ID_CAPABILITIES MSN_CLIENT_CAP_BASE | MSN_CLIENT_CAP_INK_GIF | MSN_CLIENT_CAP_VOICE_CLIP
+#else
 #define MSN_CLIENT_ID_CAPABILITIES MSN_CLIENT_CAP_BASE | MSN_CLIENT_CAP_INK_GIF
+#endif /* defined(PECAN_LIBSIREN) */
 #else
 #define MSN_CLIENT_ID_CAPABILITIES MSN_CLIENT_CAP_BASE
 #endif /* defined(PECAN_CVR) */
