@@ -81,11 +81,6 @@ open_cb (PecanNode *conn,
     session = conn->session;
     cmd_conn = CMD_PECAN_NODE (conn);
 
-    if (session->login_step == PECAN_LOGIN_STEP_START)
-        msn_session_set_login_step (session, PECAN_LOGIN_STEP_HANDSHAKE);
-    else
-        msn_session_set_login_step (session, PECAN_LOGIN_STEP_HANDSHAKE2);
-
     msn_cmdproc_send (cmd_conn->cmdproc, "VER", "MSNP12 CVR0");
 
     pecan_log ("end");
@@ -257,7 +252,7 @@ group_error_helper(MsnSession *session, const char *msg, const gchar *group_guid
     char *reason = NULL;
     char *title = NULL;
 
-    account = session->account;
+    account = msn_session_get_user_data (session);
     gc = purple_account_get_connection(account);
 
     if (error == 224)
@@ -291,8 +286,6 @@ msn_got_login_params(MsnSession *session, const char *login_params)
 
     cmdproc = session->notification->cmdproc;
 
-    msn_session_set_login_step(session, PECAN_LOGIN_STEP_AUTH_END);
-
     {
         gchar **tokens;
         tokens = g_strsplit (login_params, "&", 2);
@@ -319,14 +312,12 @@ usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     PurpleConnection *gc;
 
     session = cmdproc->session;
-    account = session->account;
+    account = msn_session_get_user_data (session);
     gc = purple_account_get_connection(account);
 
     if (!g_ascii_strcasecmp(cmd->params[1], "OK"))
     {
         /* OK */
-        msn_session_set_login_step(session, PECAN_LOGIN_STEP_SYN);
-
         msn_cmdproc_send(cmdproc, "SYN", "%s %s", "0", "0");
     }
     else if (!g_ascii_strcasecmp(cmd->params[1], "TWN"))
@@ -350,8 +341,6 @@ usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
         g_strfreev(elems);
 
-        msn_session_set_login_step(session, PECAN_LOGIN_STEP_AUTH_START);
-
         msn_nexus_connect(session->nexus);
     }
 }
@@ -613,7 +602,7 @@ adc_error(MsnCmdProc *cmdproc, MsnTransaction *trans, int error)
     char **params;
 
     session = cmdproc->session;
-    account = session->account;
+    account = msn_session_get_user_data (session);
     gc = purple_account_get_connection(account);
     params = g_strsplit(trans->params, " ", 0);
 
@@ -731,7 +720,7 @@ iln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     gchar *friendly;
 
     session = cmdproc->session;
-    account = session->account;
+    account = msn_session_get_user_data (session);
     gc = purple_account_get_connection(account);
 
     state    = cmd->params[1];
@@ -792,12 +781,12 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 #if defined(PECAN_CVR)
     MsnObject *msnobj;
 #endif /* defined(PECAN_CVR) */
-    int clientid;
+    unsigned long clientid;
     const char *state, *passport;
     gchar *friendly;
 
     session = cmdproc->session;
-    account = session->account;
+    account = msn_session_get_user_data (session);
     gc = purple_account_get_connection(account);
 
     state    = cmd->params[0];
@@ -832,17 +821,15 @@ nln_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
     }
 #endif /* defined(PECAN_CVR) */
 
-    clientid = atoi(cmd->params[3]);
+    clientid = strtoul (cmd->params[3], NULL, 10);
     user->mobile = (clientid & MSN_CLIENT_CAP_MSNMOBILE);
 
     pecan_contact_set_state(user, state);
     pecan_contact_update(user);
 
-#if 0
     /* store the friendly name on the server. */
     if (!session->server_alias)
-        msn_cmdproc_send (cmdproc, "SBP", "%s %s %s", pecan_contact_get_guid (user), "MFN", friendly);
-#endif
+        msn_cmdproc_send (cmdproc, "SBP", "%s %s %s", pecan_contact_get_guid (user), "MFN", cmd->params[2]);
 
     g_free (friendly);
 }
@@ -959,12 +946,16 @@ static void
 prp_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
     MsnSession *session = cmdproc->session;
-    PurpleConnection *gc = session->account->gc;
+    PurpleConnection *gc;
+    PurpleAccount *account;
     const gchar *type, *value;
     PecanContact *user;
 
     g_return_if_fail(cmd->param_count >= 3);
 
+    account = msn_session_get_user_data (session);
+    gc = purple_account_get_connection (account);
+
     type = cmd->params[1];
     user = msn_session_get_contact (session);
 
@@ -1179,13 +1170,15 @@ url_cmd (MsnCmdProc *cmdproc,
 {
     MsnSession *session;
     PurpleConnection *connection;
+    PurpleAccount *account;
     const gchar *rru;
     const gchar *url;
     gchar creds[64];
     glong tmp_timestamp;
 
     session = cmdproc->session;
-    connection = purple_account_get_connection (session->account);
+    account = msn_session_get_user_data (session);
+    connection = purple_account_get_connection (account);
 
     rru = cmd->params[1];
     url = cmd->params[2];
@@ -1245,7 +1238,7 @@ url_cmd (MsnCmdProc *cmdproc,
         return;
     }
 
-    if (purple_account_get_check_mail (session->account))
+    if (purple_account_get_check_mail (account))
     {
         static gboolean is_initial = TRUE;
 
@@ -1325,8 +1318,6 @@ xfr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 
         session = cmdproc->session;
 
-        msn_session_set_login_step(session, PECAN_LOGIN_STEP_TRANSFER);
-
         msn_notification_connect(session->notification, host, port);
     }
 
@@ -1390,11 +1381,9 @@ initial_mdata_msg (MsnCmdProc *cmdproc,
                    MsnMessage *msg)
 {
     MsnSession *session;
-    PurpleConnection *gc;
     GHashTable *table;
 
     session = cmdproc->session;
-    gc = session->account->gc;
 
     if (strcmp (msg->remote_user, "Hotmail"))
     {
@@ -1485,10 +1474,16 @@ initial_mdata_msg (MsnCmdProc *cmdproc,
             } while (start);
         }
 
-        if (purple_account_get_check_mail (session->account) &&
-            session->passport_info.email_enabled == 1)
         {
-            msn_cmdproc_send (cmdproc, "URL", "%s", "INBOX");
+            PurpleAccount *account;
+
+            account = msn_session_get_user_data (session);
+
+            if (purple_account_get_check_mail (account) &&
+                session->passport_info.email_enabled == 1)
+            {
+                msn_cmdproc_send (cmdproc, "URL", "%s", "INBOX");
+            }
         }
     }
 
@@ -1499,14 +1494,14 @@ static void
 email_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 {
     MsnSession *session;
-    PurpleConnection *gc;
+    PurpleAccount *account;
     GHashTable *table;
     char *from, *subject, *tmp;
 
     session = cmdproc->session;
-    gc = session->account->gc;
+    account = msn_session_get_user_data (session);
 
-    if (!purple_account_get_check_mail (session->account))
+    if (!purple_account_get_check_mail (account))
         return;
 
     if (strcmp(msg->remote_user, "Hotmail"))
@@ -1534,12 +1529,16 @@ email_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     if (tmp != NULL)
         subject = purple_mime_decode_field(tmp);
 
-    /** @todo go to the extact email */
-    purple_notify_email(gc,
-                        (subject != NULL ? subject : ""),
-                        (from != NULL ?  from : ""),
-                        msn_session_get_username (session),
-                        session->passport_info.mail_url, NULL, NULL);
+    {
+        PurpleConnection *connection;
+        connection = purple_account_get_connection (account);
+        /** @todo go to the extact email */
+        purple_notify_email (connection,
+                             (subject ? subject : ""),
+                             (from ?  from : ""),
+                             msn_session_get_username (session),
+                             session->passport_info.mail_url, NULL, NULL);
+    }
 
     g_free(from);
     g_free(subject);
@@ -1590,7 +1589,13 @@ system_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
         }
 
         if (*buf != '\0')
-            purple_notify_info(cmdproc->session->account->gc, NULL, buf, NULL);
+        {
+            PurpleAccount *account;
+            PurpleConnection *connection;
+            account = msn_session_get_user_data (cmdproc->session);
+            connection = purple_account_get_connection (account);
+            purple_notify_info (connection, NULL, buf, NULL);
+        }
     }
 
     g_hash_table_destroy(table);
