@@ -24,11 +24,16 @@
 #include "session.h"
 #include "switchboard.h"
 #include "notification.h"
+#include "fix_purple.h"
 
 #if defined(PECAN_CVR)
 #include "cvr/slplink.h"
 #endif /* defined(PECAN_CVR) */
 
+#if defined(PECAN_LIBSIREN)
+#include "pecan_siren7.h"
+#endif /* defined(PECAN_LIBSIREN) */
+
 #include "session_private.h"
 
 #include "cmd/cmdproc_private.h"
@@ -79,9 +84,6 @@ open_cb (PecanNode *conn,
 
     {
         MsnTransaction *trans;
-        PurpleAccount *account;
-
-        account = session->account;
 
         if (msn_switchboard_is_invited (swboard))
         {
@@ -360,7 +362,7 @@ msn_switchboard_add_user(MsnSwitchBoard *swboard, const char *user)
 
     g_return_if_fail(swboard);
 
-    account = swboard->session->account;
+    account = msn_session_get_user_data (swboard->session);
 
     swboard->users = g_list_prepend(swboard->users, g_strdup(user));
     swboard->current_users++;
@@ -404,7 +406,7 @@ msn_switchboard_add_user(MsnSwitchBoard *swboard, const char *user)
 
             swboard->chat_id = swboard->session->conv_seq++;
             swboard->flag |= MSN_SB_FLAG_IM;
-            swboard->conv = serv_got_joined_chat(account->gc,
+            swboard->conv = serv_got_joined_chat(purple_account_get_connection (account),
                                                  swboard->chat_id,
                                                  "MSN Chat");
 
@@ -450,17 +452,16 @@ msn_switchboard_get_conv(MsnSwitchBoard *swboard)
 {
     PurpleAccount *account;
 
-    g_return_val_if_fail(swboard, NULL);
+    g_return_val_if_fail (swboard, NULL);
 
     if (swboard->conv != NULL)
         return swboard->conv;
 
     pecan_warning ("switchboard with unassigned conversation");
 
-    account = swboard->session->account;
+    account = msn_session_get_user_data (swboard->session);
 
-    return (swboard->conv = purple_conversation_new(PURPLE_CONV_TYPE_IM,
-                                                    account, swboard->im_user));
+    return (swboard->conv = purple_conversation_new (PURPLE_CONV_TYPE_IM, account, swboard->im_user));
 }
 
 static void
@@ -532,7 +533,7 @@ msg_error_helper(MsnCmdProc *cmdproc, MsnMessage *msg, MsnMsgErrorType error)
     /* This is not good, and should be fixed somewhere else. */
     g_return_if_fail (swboard);
 
-    if (msg->type == MSN_MSG_TEXT)
+    if (msg->type == MSN_MSG_TEXT && msn_message_get_flag (msg) != 'U')
     {
         const char *format, *str_reason;
         char *body_str, *body_enc, *pre, *post;
@@ -833,36 +834,30 @@ bye_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 }
 
 static void
-iro_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
+iro_cmd (MsnCmdProc *cmdproc,
+         MsnCommand *cmd)
 {
-    PurpleAccount *account;
-    PurpleConnection *gc;
     MsnSwitchBoard *swboard;
 
-    account = cmdproc->session->account;
-    gc = account->gc;
     swboard = cmdproc->data;
+
     g_return_if_fail (swboard);
 
-    swboard->total_users = atoi(cmd->params[2]);
+    swboard->total_users = atoi (cmd->params[2]);
 
-    msn_switchboard_add_user(swboard, cmd->params[3]);
+    msn_switchboard_add_user (swboard, cmd->params[3]);
 }
 
 static void
 joi_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
     MsnSession *session;
-    PurpleAccount *account;
-    PurpleConnection *gc;
     MsnSwitchBoard *swboard;
     const char *passport;
 
     passport = cmd->params[0];
 
     session = cmdproc->session;
-    account = session->account;
-    gc = account->gc;
     swboard = cmdproc->data;
     g_return_if_fail (swboard);
 
@@ -945,15 +940,19 @@ ack_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 static void
 out_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 {
-    PurpleConnection *gc;
     MsnSwitchBoard *swboard;
 
-    gc = cmdproc->session->account->gc;
     swboard = cmdproc->data;
     g_return_if_fail (swboard);
 
     if (swboard->current_users > 1)
-        serv_got_chat_left(gc, swboard->chat_id);
+    {
+        PurpleAccount *account;
+        PurpleConnection *connection;
+        account = msn_session_get_user_data (cmdproc->session);
+        connection = purple_account_get_connection (account);
+        serv_got_chat_left (connection, swboard->chat_id);
+    }
 
     msn_switchboard_disconnect(swboard);
 }
@@ -993,20 +992,107 @@ usr_cmd(MsnCmdProc *cmdproc, MsnCommand *cmd)
 /**************************************************************************
  * Message Handlers
  **************************************************************************/
+
+static void
+got_datacast_inform_user (MsnCmdProc *cmdproc,
+                          const char *passport,
+                          const char *str)
+{
+    PurpleAccount *account;
+    MsnSwitchBoard *swboard;
+    PecanContact *contact;
+    const char *friendly_name;
+    gchar *new_str;
+
+    account = msn_session_get_user_data (cmdproc->session);
+    swboard = cmdproc->data;
+    contact = pecan_contactlist_find_contact(cmdproc->session->contactlist, passport);
+    friendly_name = pecan_contact_get_friendly_name(contact);
+    if (!friendly_name)
+        friendly_name = passport;
+
+    new_str = g_strdup_printf("%s %s", friendly_name, str);
+
+    /* Grab the conv for this swboard. If there isn't one and it's an IM then create it,
+    otherwise the smileys won't work, this needs to be fixed. */
+    if (!swboard->conv)
+    {
+        if (swboard->current_users > 1)
+            swboard->conv = purple_find_chat(account->gc, swboard->chat_id);
+        else
+        {
+            swboard->conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM,
+                                                                  passport, account);
+            if (!swboard->conv)
+                swboard->conv = purple_conversation_new(PURPLE_CONV_TYPE_IM, account, passport);
+        }
+    }
+    swboard->flag |= MSN_SB_FLAG_IM;
+
+    purple_conversation_write(swboard->conv, NULL, new_str, PURPLE_MESSAGE_SYSTEM, time(NULL));
+
+    g_free (new_str);
+}
+
+#if defined(RECEIVE_PLUS_SOUNDS)
+static void
+save_plus_sound_cb(PurpleUtilFetchUrlData *url_data, gpointer user_data,
+                   const gchar *sound, size_t len, const gchar *error_message)
+{
+    FILE *f;
+    gchar *path;
+    gchar *str;
+    MsnCmdProc *cmdproc = user_data;
+    const char *passport = cmdproc->extra_data;
+
+    if ((error_message != NULL) || (len == 0))
+        return;
+
+    if (purple_mkstemp(&path, TRUE))
+    {
+        gchar *path_mp3 = g_strconcat (path, ".mp3", NULL);
+
+        f = fopen (path_mp3, "wb");
+        fwrite(sound, len, 1, f);
+
+        str = g_strdup_printf (_("sent you a Messenger Plus! sound. Click <a href='file://%s'>here</a> to play it."), path_mp3);
+        got_datacast_inform_user (cmdproc, passport, str);
+
+        fclose(f);
+
+        g_free (path_mp3);
+    }
+    else
+    {
+        pecan_error ("couldn't create temporany file to store the received Plus! sound!\n");
+
+        str = g_strdup_printf (_("sent you a Messenger Plus! sound, but it cannot be played due to an error happened while storing the file."));
+        got_datacast_inform_user (cmdproc, passport, str);
+    }
+
+    g_free(str);
+    g_free(path);
+}
+#endif /* defined(RECEIVE_PLUS_SOUNDS) */
+
 static void
 plain_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 {
     PurpleConnection *gc;
+    PurpleAccount *account;
     MsnSwitchBoard *swboard;
+    PurpleBuddy *buddy;
     const char *body;
     char *body_str;
     char *body_enc;
     char *body_final;
+    char *alias_backup = NULL;
     size_t body_len;
-    const char *passport;
+    char *passport;
     const char *value;
 
-    gc = cmdproc->session->account->gc;
+    account = msn_session_get_user_data (cmdproc->session);
+    gc = purple_account_get_connection (account);
     swboard = cmdproc->data;
     g_return_if_fail (swboard);
 
@@ -1015,7 +1101,9 @@ plain_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     body_enc = g_markup_escape_text(body_str, -1);
     g_free(body_str);
 
-    passport = msg->remote_user;
+    passport = strdup (msg->remote_user);
+
+    buddy = purple_find_buddy(account, passport);
 
     if (!strcmp(passport, "messenger@microsoft.com") &&
         strstr(body, "immediate security update"))
@@ -1030,6 +1118,12 @@ plain_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     }
 #endif
 
+    if ((value = msn_message_get_attr(msg, "P4-Context")) != NULL)
+    {
+        alias_backup = g_strdup(buddy->alias);
+        purple_buddy_set_displayname(gc, passport, value);
+    }
+
     if ((value = msn_message_get_attr(msg, "X-MMS-IM-Format")) != NULL)
     {
         char *pre, *post;
@@ -1050,35 +1144,67 @@ plain_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 
     swboard->flag |= MSN_SB_FLAG_IM;
 
-    if (swboard->current_users > 1 ||
-        ((swboard->conv != NULL) &&
-         purple_conversation_get_type(swboard->conv) == PURPLE_CONV_TYPE_CHAT))
-    {
-        /* If current_users is always ok as it should then there is no need to
-         * check if this is a chat. */
-        if (swboard->current_users <= 1)
-            pecan_warning ("plain_msg: current_users=[%d]", swboard->current_users);
+#if defined(RECEIVE_PLUS_SOUNDS)
+    const char *body_plus_sound = strstr (body_final, "[Messenger Plus! Sound] - Data{");
 
-        serv_got_chat_in(gc, swboard->chat_id, passport, 0, body_final,
-                         time(NULL));
-        if (swboard->conv == NULL)
+    if (body_plus_sound && strlen (body_plus_sound) > 43)
+    {
+        char *plus_sound_link = calloc (48 + 12 + 1, 1);
+
+        strcpy (plus_sound_link, "http://sounds.msgpluslive.net/esnd/snd/get?hash=");
+        strncat (plus_sound_link, body_plus_sound + 31, 12);
+
+        cmdproc->extra_data = passport;
+
+        purple_util_fetch_url (plus_sound_link, TRUE, NULL, FALSE, save_plus_sound_cb, cmdproc);
+
+        free (plus_sound_link);
+
+        return;
+    }
+#endif /* defined(RECEIVE_PLUS_SOUNDS) */
+
+    {
+        PurpleAccount *account;
+        PurpleConnection *connection;
+
+        account = msn_session_get_user_data (cmdproc->session);
+        connection = purple_account_get_connection (account);
+
+        if (swboard->current_users > 1 ||
+            (swboard->conv && purple_conversation_get_type (swboard->conv) == PURPLE_CONV_TYPE_CHAT))
         {
-            swboard->conv = purple_find_chat(gc, swboard->chat_id);
-            swboard->flag |= MSN_SB_FLAG_IM;
+            /* If current_users is always ok as it should then there is no need to
+             * check if this is a chat. */
+            if (swboard->current_users <= 1)
+                pecan_warning ("plain_msg: current_users=[%d]", swboard->current_users);
+
+            serv_got_chat_in (connection, swboard->chat_id, passport, 0, body_final, time (NULL));
+            if (!swboard->conv)
+            {
+                swboard->conv = purple_find_chat (connection, swboard->chat_id);
+                swboard->flag |= MSN_SB_FLAG_IM;
+            }
+        }
+        else
+        {
+            serv_got_im (connection, passport, body_final, 0, time (NULL));
+            if (!swboard->conv)
+            {
+                swboard->conv = purple_find_conversation_with_account (PURPLE_CONV_TYPE_IM, passport, account);
+                swboard->flag |= MSN_SB_FLAG_IM;
+            }
         }
     }
-    else
+
+    if (alias_backup)
     {
-        serv_got_im(gc, passport, body_final, 0, time(NULL));
-        if (swboard->conv == NULL)
-        {
-            swboard->conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM,
-                                                                  passport, purple_connection_get_account(gc));
-            swboard->flag |= MSN_SB_FLAG_IM;
-        }
+        purple_buddy_set_displayname(gc, passport, alias_backup);
+        g_free(alias_backup);
     }
 
     g_free(body_final);
+    g_free(passport);
 }
 
 #if defined(PECAN_CVR)
@@ -1088,18 +1214,21 @@ msn_handwritten_msg_show(MsnSwitchBoard *swboard, const char* msgid, const char*
     guchar *guc;
     size_t body_len;
     PurpleAccount *account;
+    PurpleConnection *connection;
     
-    account = swboard->session->account;
+    account = msn_session_get_user_data (swboard->session);
+    connection = purple_account_get_connection (account);
+
     guc = purple_base64_decode(data, &body_len);
     if (!guc || !body_len) 
         return;
     
-    /*Grab the convo for this sboard. If there isn't one and it's an IM 
-        then create it, otherwise the smileys won't work*/
+    /* Grab the conv for this swboard. If there isn't one and it's an IM then create it,
+    otherwise the smileys won't work, this needs to be fixed. */
     if (swboard->conv == NULL)
     {
         if (swboard->current_users > 1) 
-            swboard->conv = purple_find_chat(account->gc, swboard->chat_id);
+            swboard->conv = purple_find_chat(connection, swboard->chat_id);
         else
         {
             swboard->conv = purple_find_conversation_with_account(PURPLE_CONV_TYPE_IM,
@@ -1110,26 +1239,22 @@ msn_handwritten_msg_show(MsnSwitchBoard *swboard, const char* msgid, const char*
     }
     swboard->flag |= MSN_SB_FLAG_IM;
 
-    /*Create the image as a custom smiley in the conversation with the GUID of the message that 
-        the image arrived in as the smiley text.*/
     if (purple_conv_custom_smiley_add(swboard->conv, msgid, 0, "", 0)) {
         purple_conv_custom_smiley_write(swboard->conv, msgid, guc, body_len);
         purple_conv_custom_smiley_close(swboard->conv, msgid);
     }
     
-    /*And put the GUID into the convo window so that it will display...*/
     if (swboard->current_users > 1 ||
         ((swboard->conv != NULL) &&
          purple_conversation_get_type(swboard->conv) == PURPLE_CONV_TYPE_CHAT))
-        serv_got_chat_in(account->gc, swboard->chat_id, passport, 0, msgid,
+        serv_got_chat_in(connection, swboard->chat_id, passport, 0, msgid,
                          time(NULL));
     else
-        serv_got_im(account->gc, passport, msgid, 0, time(NULL));
+        serv_got_im(connection, passport, msgid, 0, time(NULL));
 
     g_free(guc);
 }
 
-/*only called from chats. Handwritten messages for IMs come as a SLP message*/
 static void
 msn_handwritten_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 {
@@ -1140,8 +1265,6 @@ msn_handwritten_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
     passport = msg->remote_user;
     msgid = msn_message_get_attr(msg, "Message-ID");
 
-    pecan_debug ("Displaying handwritten message from message %s", msgid);
-
     body = msn_message_get_bin_data(msg, &body_len);
     bodydup = g_strndup(body+7, body_len-7);
     msn_handwritten_msg_show(cmdproc->data, msgid, bodydup, passport);
@@ -1152,51 +1275,104 @@ msn_handwritten_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 static void
 control_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
 {
-    PurpleConnection *gc;
     MsnSwitchBoard *swboard;
     char *passport;
 
-    gc = cmdproc->session->account->gc;
     swboard = cmdproc->data;
     passport = msg->remote_user;
 
     g_return_if_fail (swboard);
+
     if (swboard->current_users == 1 &&
         msn_message_get_attr(msg, "TypingUser") != NULL)
     {
-        serv_got_typing(gc, passport, MSN_TYPING_RECV_TIMEOUT,
-                        PURPLE_TYPING);
+        PurpleAccount *account;
+        PurpleConnection *connection;
+        account = msn_session_get_user_data (cmdproc->session);
+        connection = purple_account_get_connection (account);
+        serv_got_typing (connection, passport, MSN_TYPING_RECV_TIMEOUT, PURPLE_TYPING);
     }
 }
 
+#if defined(PECAN_LIBSIREN)
 static void
-clientcaps_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
+got_voice_clip(MsnSlpCall *slpcall, const guchar *data, gsize size)
 {
-#if 0
-    MsnSession *session;
-    MsnSwitchBoard *swboard;
-    PecanContact *user;
-    GHashTable *clientcaps;
-    const char *value;
+    FILE *f;
+    char *file;
+    gchar *str;
 
-    char *passport = msg->sender;
+    if ((f = purple_mkstemp(&file, TRUE)))
+    {
+        gchar *decoded_file;
 
-    session = cmdproc->session;
+        fwrite(data, size, 1, f);
+        fclose(f);
 
-    clientcaps = msn_message_get_hashtable_from_body(msg);
-#endif
+        decoded_file = g_strconcat (file, "_decoded.wav", NULL);
+
+        decode_wav_using_siren7 (file, decoded_file);
+
+        str = g_strdup_printf(_("sent you a voice clip. Click <a href='file://%s'>here</a> to play it."), decoded_file);
+        got_datacast_inform_user(slpcall->slplink->swboard->cmdproc, slpcall->slplink->remote_user, str);
+
+        g_free (decoded_file);
+    } else {
+        pecan_error ("couldn't create temporany file to store the received voice clip!\n");
+
+        str = g_strdup_printf(_("sent you a voice clip, but it cannot be played due to an error happened while storing the file."));
+        got_datacast_inform_user(slpcall->slplink->swboard->cmdproc, slpcall->slplink->remote_user, str);
+    }
+
+    g_free (str);
 }
+#endif /* defined(PECAN_LIBSIREN) */
 
 static void
-nudge_msg(MsnCmdProc *cmdproc, MsnMessage *msg)
+datacast_msg (MsnCmdProc *cmdproc,
+              MsnMessage *msg)
 {
     PurpleAccount *account;
-    const char *user;
+    PurpleConnection *connection;
+    GHashTable *body;
+    const char *id, *passport;
+    body = msn_message_get_hashtable_from_body(msg);
 
-    account = cmdproc->session->account;
-    user = msg->remote_user;
+    id = g_hash_table_lookup(body, "ID");
 
-    serv_got_attention(account->gc, user, MSN_NUDGE);
+    account = msn_session_get_user_data (cmdproc->session);
+    connection = purple_account_get_connection (account);
+    passport = msg->remote_user;
+
+    if (strcmp (id, "1") == 0)
+    {
+        serv_got_attention (connection, passport, MSN_NUDGE);
+    }
+    else if (strcmp (id, "2") == 0)
+    {
+        /* winks */
+    }
+    else if (strcmp (id, "3") == 0)
+    {
+#if defined(PECAN_LIBSIREN)
+        const char *data;
+        MsnSlpLink *slplink;
+        MsnObject *obj;
+
+        data = g_hash_table_lookup(body, "Data");
+        obj = msn_object_new_from_string(data);
+        slplink = msn_session_get_slplink(cmdproc->session, passport);
+
+        msn_slplink_request_object(slplink, data, got_voice_clip, NULL, obj);
+            
+        msn_object_destroy(obj);
+#endif /* defined(PECAN_LIBSIREN) */
+    }
+    else
+    {
+        pecan_warning ("Got unknown datacast with ID %s.\n", id);
+        serv_got_attention (connection, passport, MSN_NUDGE);
+    }
 }
 
 /**************************************************************************
@@ -1484,10 +1660,6 @@ msn_switchboard_init(void)
                            plain_msg);
     msn_table_add_msg_type(cbs_table, "text/x-msmsgscontrol",
                            control_msg);
-    msn_table_add_msg_type(cbs_table, "text/x-clientcaps",
-                           clientcaps_msg);
-    msn_table_add_msg_type(cbs_table, "text/x-clientinfo",
-                           clientcaps_msg);
 #if defined(PECAN_CVR)
     msn_table_add_msg_type(cbs_table, "application/x-msnmsgrp2p",
                            msn_p2p_msg);
@@ -1499,7 +1671,7 @@ msn_switchboard_init(void)
                            msn_handwritten_msg);
 #endif /* defined(PECAN_CVR) */
     msn_table_add_msg_type(cbs_table, "text/x-msnmsgr-datacast",
-                           nudge_msg);
+                           datacast_msg);
 #if 0
     msn_table_add_msg_type(cbs_table, "text/x-msmmsginvite",
                            msn_invite_msg);
